@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2010 OpenVPN Technologies, Inc. <sales@openvpn.net>
+ *  Copyright (C) 2002-2017 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -840,11 +840,18 @@ redirect_default_route_to_vpn (struct route_list *rl, const struct tuntap *tt, u
 
   if ( rl && rl->flags & RG_ENABLE )
     {
+      bool local = BOOL_CAST(rl->flags & RG_LOCAL);
       if (!(rl->spec.flags & RTSA_REMOTE_ENDPOINT) && (rl->flags & RG_REROUTE_GW))
 	{
 	  msg (M_WARN, "%s VPN gateway parameter (--route-gateway or --ifconfig) is missing", err);
 	}
-      else if (!(rl->rgi.flags & RGI_ADDR_DEFINED))
+      /*
+       * check if a default route is defined, unless:
+       * - we are connecting to a remote host in our network
+       * - we are connecting to a non-IPv4 remote host (i.e. we use IPv6)
+       */
+      else if (!(rl->rgi.flags & RGI_ADDR_DEFINED) && !local
+	       && (rl->spec.remote_host != IPV4_INVALID_ADDR))
 	{
 	  msg (M_WARN, "%s Cannot read current default gateway from system", err);
 	}
@@ -854,7 +861,6 @@ redirect_default_route_to_vpn (struct route_list *rl, const struct tuntap *tt, u
 	}
       else
 	{
-	  bool local = BOOL_CAST(rl->flags & RG_LOCAL);
 	  if (rl->flags & RG_AUTO_LOCAL) {
 	    const int tla = rl->spec.remote_host_local;
 	    if (tla == TLA_NONLOCAL)
@@ -914,14 +920,18 @@ redirect_default_route_to_vpn (struct route_list *rl, const struct tuntap *tt, u
 		}
 	      else
 		{
-		  /* delete default route */
-		  del_route3 (0,
-			      0,
-			      rl->rgi.gateway.addr,
-			      tt,
-			      flags | ROUTE_REF_GW,
-			      &rl->rgi,
-			      es);
+		  /* don't try to remove the def route if it does not exist */
+		  if (rl->rgi.flags & RGI_ADDR_DEFINED)
+		  {
+		      /* delete default route */
+		      del_route3 (0,
+				  0,
+				  rl->rgi.gateway.addr,
+				  tt,
+				  flags | ROUTE_REF_GW,
+				  &rl->rgi,
+				  es);
+		  }
 
 		  /* add new default route */
 		  add_route3 (0,
@@ -994,14 +1004,18 @@ undo_redirect_default_route_to_vpn (struct route_list *rl, const struct tuntap *
 			  &rl->rgi,
 			  es);
 
-	      /* restore original default route */
-	      add_route3 (0,
-			  0,
-			  rl->rgi.gateway.addr,
-			  tt,
-			  flags | ROUTE_REF_GW,
-			  &rl->rgi,
-			  es);
+	      /* restore original default route if there was any */
+	      if (rl->rgi.flags & RGI_ADDR_DEFINED)
+	        {
+		  /* restore original default route */
+		  add_route3 (0,
+			      0,
+			      rl->rgi.gateway.addr,
+			      tt,
+			      flags | ROUTE_REF_GW,
+			      &rl->rgi,
+			      es);
+	        }
 	    }
 	}
 
@@ -3009,6 +3023,9 @@ get_default_gateway (struct route_gateway_info *rgi)
   rtm.rtm_flags = RTF_UP | RTF_GATEWAY;
   rtm.rtm_version = RTM_VERSION;
   rtm.rtm_seq = ++seq;
+#ifdef TARGET_OPENBSD
+  rtm.rtm_tableid = getrtable();
+#endif
   rtm.rtm_addrs = rtm_addrs; 
 
   so_dst.sa_family = AF_INET;
